@@ -1,26 +1,22 @@
-/* 
+/*
    This file is part of GNU Sqltutor
-   Copyright (C) 2008  Free Software Foundation, Inc.
+   Copyright (C) 2008, 2013  Free Software Foundation, Inc.
    Contributed by Ales Cepek <cepek@gnu.org>
- 
+
    GNU Sqltutor is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
-   
+
    GNU Sqltutor is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with GNU Sqltutor.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* 
- * $Id: form_init.cpp,v 1.7 2009/04/01 18:12:37 cepek Exp $ 
- */
-
 #include <pqxx/pqxx>
 #include "cgi.h"
 #include "sqltutor.h"
@@ -32,14 +28,14 @@ namespace
 {
   string emptyrow() { return "<tr><td>&nbsp;</td></tr>";    }
   string emptycol() { return "<td>&nbsp;&nbsp;&nbsp;</td>"; }
-  
+
   bool number(string s)
   {
-    if (s.empty()) return true;
-    
     string::const_iterator b=s.begin(), e=s.end();
     while (b != e && isspace(*b)) b++;
-    
+
+    if (b == e) return true;
+
     int count = 0;
     while (b != e)
       {
@@ -48,17 +44,18 @@ namespace
         b++;
       }
     if (count == 0 || count > 5) return false;  // number too big
-    
+
     while (b != e)
       {
         if (!isspace(*b)) return false;
         b++;
       }
-    
+
     return true;
   }
 
-  string param(const string& ap, const string& val)
+  string param(const string& ap, const string& val,
+               const string& implicit_value="NULL")
   {
     bool empty = true;
     for (string::const_iterator b=val.begin(), e=val.end(); b!=e; ++b)
@@ -67,7 +64,7 @@ namespace
           empty = false;
           break;
         }
-    if (empty) return "NULL";
+    if (empty) return implicit_value;
 
     return ap + val + ap;
   }
@@ -76,55 +73,102 @@ namespace
 
 void SQLtutor::form_init()
 {
-  const string state    = CGI::map["state"];
+  string state          = CGI::map["state"];
   const string user     = CGI::map["user"];
   const string tutorial = CGI::map["tutorial"];
 
   bool   error = false;
   string pmin  = CGI::map["points_min"];
   string pmax  = CGI::map["points_max"];
-  string dset  = CGI::map["dataset"];
+  string dset, tmp = CGI::map["dataset"];
+  for (string::const_iterator b=tmp.begin(), e=tmp.end(); b!=e; b++)
+    if (!isspace(*b))
+      {
+        dset += *b;
+      }
+
+  Table perr(2);
 
   if (!number(pmin))
     {
       error = true;
-      form << t_bad_value_min << ": " << pmin << " <br/>";
+      perr << t_bad_value_min << pmin;
     }
   if (!number(pmax))
     {
       error = true;
-      form << t_bad_value_max << ": " << pmax << " <br/>";
+      perr << t_bad_value_max << pmax;
     }
-  
+
+  bool dollar = false;
+  if (!dset.empty())
+    {
+      for (string::const_iterator b=dset.begin(), e=dset.end(); b!=e; b++)
+        if (*b == '$' || *b == '&')
+          {
+            dollar = true;
+            error = true;
+            perr << t_bad_value_char << dset;
+            break;
+          }
+    }
+
+  if (!user.empty())
+    {
+      for (string::const_iterator b=user.begin(), e=user.end(); b!=e; b++)
+        if (*b == '$' || *b == '&')
+          {
+            error = true;
+            perr << t_bad_value_char << user;
+            break;
+          }
+    }
+
+  if (!dollar && !dset.empty() && tutorial != "0")
+    {
+      using namespace pqxx;
+      string str = "$sqltutor$";
+      string sql =
+        "SELECT dataset_id "
+        "FROM   datasets AS d "
+        "JOIN   tutorials_problems AS tp USING (dataset_id) "
+        "JOIN   tutorials AS t USING (tutorial_id) "
+        "WHERE  dataset = " + str + dset + str + " "
+        "AND    tutorial_id = '" + tutorial + "' ";
+      connection conn( db_connection );
+      work   tran(conn, "check_dataset");
+      set_schema(tran);
+
+      result sql_result(tran.exec(sql));
+      if (sql_result.empty())
+        {
+          error = true;
+          perr << t_bad_value_dset << dset;
+        }
+    }
+
   if (error)
     {
-      CGI::map["state"] =  init_state;
-      CGI::map["user"]  = "";
-      
-      form << "<br/>";
-      form_init();
-      return;
+      CGI::map["state"] = init_state;
     }
-  
-  if (state == init_continue && tutorial != "0")
-    try 
-      {
-        string help; 
-        if (!CGI::map["help"].empty())
-          help = "true";
-        else
-          help = "false";
 
-        const string open = 
+  if (!error && state == init_continue && tutorial != "0")
+    try
+      {
+        // checked by form_main()
+        if (CGI::map["help"].empty()) CGI::map["help"] = "0";
+
+        const string ihelp = (CGI::map["help"] == "true") ? "1" : "0";
+        const string open =
           "SELECT session_id_, hash_ FROM open_session (" +
-          param(" ", tutorial                  ) + ", " +   
-          param("'", CGI::map["user"]          ) + ", " +   
-          param("'", CGI::map["password"]      ) + ", " +  
-          param(" ", CGI::map["points_min"]    ) + ", " +  
-          param(" ", CGI::map["points_max"]    ) + ", " +  
-          param("'", CGI::map["dataset"]       ) + ", " +  
-          param(" ", help                      ) + ", " +  
-          param("'", CGI::getenv("REMOTE_ADDR")) + ");";
+          param(" ", tutorial                    ) + ", " +
+          param("'", CGI::map["user"]            ) + ", " +
+          param("'", CGI::map["password"]        ) + ", " +
+          param(" ", CGI::map["points_min"], "0" ) + ", " +
+          param(" ", CGI::map["points_max"], "0" ) + ", " +
+          param("'", CGI::map["dataset"]         ) + ", " +
+          param(" ", ihelp,                  "0" ) + ", " +
+          param("'", CGI::getenv("REMOTE_ADDR")  ) + ");";
 
         using namespace pqxx;
         connection  conn( db_connection );
@@ -132,68 +176,78 @@ void SQLtutor::form_init()
         set_schema(tran);
         result res1(tran.exec(open));
         result::const_iterator q = res1.begin();
-        if (q == res1.end()) throw "...";
-        string session_id  = q[0].as(string());
-        string hash        = q[1].as(string());
-        tran.commit();
-        
-        CGI::map.clear();
-        CGI::map["session_id"] = session_id;
-        CGI::map["state"]      = main_next;
-        CGI::map["hash"]       = hash;
-        CGI::map["help"]       = help;            // checked by form_main() 
+        if (q == res1.end())
+          {
+            error = true;
+            perr << "Login failed for user ";
+            if (user.empty())
+              perr << "...";
+            else
+              perr << user;
+          }
+        else
+          {
+            string session_id  = q[0].as(string());
+            string hash        = q[1].as(string());
+            tran.commit();
 
-        return form_main();
+            CGI::map.clear();
+            CGI::map["session_id"] = session_id;
+            CGI::map["state"]      = main_next;
+            CGI::map["hash"]       = hash;
+
+            return form_main();
+          }
       }
-    catch (const std::exception& s) 
+    catch (const std::exception& s)
       {
         form << s.what() << "<br/><br/>";
         form_stop();
         return;
       }
 
-  Par welcome;
-  welcome << "<strong>" << t_welcome << "</strong>";
-  form    << welcome;
+  Strong welcome;
+  welcome << t_welcome;
+  form    << (Par() << welcome);
 
   form << "<table>";
-  form << "<tr>" 
-       << "<td>" + t_tutorial + "&nbsp;</td>"
+  form << "<tr>"
+       << "<td>" + t_tutorial + "</td>"
        << "<td>"
        << tutorial_selection()
        << "</td>"
        << "</tr>";
   form << emptyrow();
   form << "<tr>"
-       << "<td>" + t_user + "&nbsp;</td>" 
-       << "<td>" 
-       << InputText("user").value("guest").disabled()
+       << "<td>" + t_user + "</td>"
+       << "<td>"
+       << InputText("user").value(user)
        << "</td>"
        << emptycol()
-       << "<td>" + t_password + "&nbsp;</td>"
+       << "<td>" + t_password + "</td>"
        << "<td>"
-       << InputPassword("password").value("").disabled()
+       << InputPassword("password").value("").disabled(true)
        << "</td>"
        << "</tr>";
   form << emptyrow();
   form << "<tr>"
-       << "<td>" + t_points_min + "&nbsp;</td>"
+       << "<td>" + t_points_min + "</td>"
        << "<td>"
        << InputText("points_min").value(pmin)
        << "</td>"
        << emptycol()
-       << "<td>" + t_points_max + "&nbsp;</td>" 
+       << "<td>" + t_points_max + "</td>"
        << "<td>"
        << InputText("points_max").value(pmax)
        << "</td>"
        << "</tr>";
   form << "<tr>"
-       << "<td>" + t_dataset + "&nbsp;</td>"
+       << "<td>" + t_dataset + "</td>"
        << "<td>"
-       << InputText("dataset").value(dset) 
-       << "</td>" 
+       << InputText("dataset").value(dset)
+       << "</td>"
        << emptycol()
-       << "<td>" + t_help + "&nbsp;</td>"
+       << "<td>" + t_help + "</td>"
        << "<td>";
 
   form << InputCheckbox("help").value("true").checked(!CGI::map["help"].empty());
@@ -211,6 +265,18 @@ void SQLtutor::form_init()
        << InputSubmit("state").value(init_manual);
   form << "</p>";
 
+  if (tutorial == "0" && (state == init_datasets || state == init_continue))
+    {
+      form << "<p>"
+           << t_select_tutorial
+           << "</p>";
+    }
+
+  if (error)
+    {
+      form << (Par() << perr);
+    }
+
   const string prev_state = CGI::map["prev_state"];
 
   if (prev_state != init_datasets && state == init_datasets && tutorial != "0")
@@ -225,10 +291,4 @@ void SQLtutor::form_init()
       getting_started();
     }
 
-  if (tutorial == "0" && (state == init_datasets || state == init_continue))
-    {
-      form << "<p>"
-           << t_select_tutorial
-           << "</p>";
-    }
 }
