@@ -1,6 +1,6 @@
 /* 
    This file is part of GNU Sqltutor
-   Copyright (C) 2008  Free Software Foundation, Inc.
+   Copyright (C) 2008, 2010  Free Software Foundation, Inc.
    Contributed by Ales Cepek <cepek@gnu.org>
  
    GNU Sqltutor is free software: you can redistribute it and/or modify
@@ -17,10 +17,6 @@
    along with GNU Sqltutor.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* 
- * $Id: show_table_data.cpp,v 1.6 2009/04/01 18:12:38 cepek Exp $ 
- */
-
 #include "sqltutor.h"
 
 using std::string;
@@ -32,24 +28,18 @@ void SQLtutor::show_table_data(pqxx::work& tran)
 try 
 {
   result tables(tran.exec("SELECT ds_table, columns "
-                          "  FROM questions Q "
-                          "       JOIN "
-                          "       datasets  D "
-                          "       ON Q.dataset = D.dataset "
-                          "          AND tutorial_id = " + tutorial_id +
-                          "          AND id='" + question_id + "' "
-                          " ORDER BY ord; "));
+                          "  FROM questions"
+                          "       NATURAL JOIN dataset_tables"
+                          " WHERE dataset_id  ='" + dataset_id  + "' "
+                          "   AND problem_id  ='" + problem_id  + "' "
+                          "   AND q_ord       ='" + q_ord       + "' "
+                          "   AND language_id ='" + language_id + "' "
+                          " ORDER BY dt_ord; "));
   
   for (result::const_iterator t=tables.begin(), e=tables.end(); t!=e; t++)
     {
       const string table = t[0].as(string());
 
-      // IE6 doesn't honor the colspan="0", with or without a colgroup
-      // defined
-      // 
-      // form << "<p></p><table border='1'>"
-      //      << "<tr><th colspan='0'>" + table + "</th></tr>";
-
       int icols = 1;
       string columns = "<tr><th><i>";
       const string col = t[1].as(string());
@@ -65,25 +55,54 @@ try
            << "<tr><th colspan='" << icols << "'>" + table + "</th></tr>";
       form << columns;
 
-      {
-        result data(tran.exec("SELECT * FROM " + table));
+      try {
+        result data(tran.exec("SELECT * FROM " + table + " ORDER BY random()"));
+        size_t row_count = 0, total_count=data.size();
         size_t columns = data.columns();
         for (result::const_iterator d=data.begin(), e=data.end(); d!=e; d++)
           {
+            if (++row_count > max_row_count && row_count != total_count)
+              {
+                form << "<tr><td colspan='" << columns << "'>"
+                     << "<i>" << remaining_rows << "</i></td></tr>";
+          break;
+              }
             form << "<tr>";
             for (size_t i=0; i<columns; i++)
-              {
-                form << "<td>" << d[i].as(string()) << "</td>";
-              }
+	      if (d[i].type() == geom_oid)  // postgis geometry
+		{
+		  form << "<td>" << t_geometry << "</td>";
+		}
+	      else
+		{
+		  form << "<td>" << d[i].as(string()) << "</td>";
+		}
             form << "</tr>";
           }
       }
+      catch (sql_error e)
+        {
+          form << e.what();
+        }
+      catch (std::exception& e)
+        {
+          form << e.what();
+        }
+      catch (...)
+        {
+          throw;
+        }
 
       form << "</table>";
     }
 }
-catch (sql_error)
+catch (sql_error e)
 {
+  form << e.what();
+}
+catch (std::exception& e)
+{
+  form << e.what();
 }
 catch (...)
 {
